%%
% function FMBS_init() is the function that initializes the GUI. It creates
% the Lawrence diagram, initializes the to be designed controller, control
% architecture, discretisation settings, defining dynamics and the
% stability criterion.
%
%
%   global data contains 
%
%   stabilitycriterion:     Which stabilitycriterion is currently defined
%   designcontrollers:      Which controller is unknown and does the user want
%                           to design
%   controlarchitecture:    Which controlarchitecture does the user want to
%                           use
%   s:                      La Place variable tf('s')
%   screensize:             The screensize of the userscreen
%   handles:                The handles to every element in the GUI
%   discretisation:         All discretisation settings
%   dynamics:               All defined dynamics and controllers
%   parameters:             Freq,Mag and Phase vectors corresponding to
%                           discretisation settings
%   stabilityproperties:    Matrix (nFreq,nMag,nPhase) with 1,0 indicating
%                           stability properties
%   temp:                   Temporary saved data
%


function FMBS_init(action,hObject,eventdata,handles)
global data

%% Initialization variables
switch action
    case 'start'        
        s = tf('s');
        screensize = get(0,'MonitorPositions');
        data.s = s;
        data.screensize = [screensize(1,3), screensize(1,4)];
        data.handles = handles;
        createLawrence();
        data.colourbutton = [0.94 0.94 0.94];
        data.plotColour = [ones(1,3)*0.6; ones(1,3)*0.8];
        data.Name = 'Settings';
        data.ListControllers = {'C1'};
        data.stepOfAction = [1, 1];
        data.mapped = 0;
        data.CurController = 1;
        data.Controller = {};
        data.symController = {};
        data.Design = {};
        data.plot.Magcontroller = {};
        data.plot.Phasecontroller = {};
        data.plot.Pointcontroller = {};
        clear data.stabilityproperties       
        data.plotController = 1;
        initDiscretisation('init');
        initDefineDynamics();
        initFigures();               
        initHandles();
        data.DesignNaming = 1;
        data.strElements = {'gainedit','gaintext','unitgain','zerotext','zeroedit','unitzero',...
            'poletext','poleedit','unitpole','dampzerotext','dampzeroedit','unitdampzero',...
            'damppoletext','damppoleedit','unitdamppole'};      
        
    case 'designcontrollers'
        data.designcontrollers = 'C1';        
    case 'controlarchitecture'
        data.controlarchitecture = '4Channel';        
    case 'stabilitycriterion'         
        data.stabilitycriterion = 'Default';        
    case 'closeFMBS'
        clear all; close all;clc;       
end

end






