%%
%   function controlarchitectures()
%
%   This function identifies the chosen control architecture and the to
%   be designed controller. The to be designed controller that is selected
%   is coloured green. The control architecture that is selected shows all
%   corresponding controllers and disables unused controllers
%
%   Input:        action: a string that indicates the action that should be
%                         performed: (control architectures or designcontrollers)
%
%   global data includes the handles to the elements in the GUI, the
%   identified controlarchitecture and identified to be designed
%   controller
%
%%
function controlarchitectures(action,hObject,eventdata,handles)
global data;

%% Initialization variables

switch action
    case 'controlarchitecture'
        % Evaluate action at the control architecture Popup Menu
        
        % Read Out Popup Menu
        controlArchitecture = cellstr(get(hObject,'String'));
        controlArchitecture = controlArchitecture{get(hObject,'Value')};
        
        set(data.handles.(data.designcontrollers),'BackgroundColor',data.colourbutton)
        
        
        j = ['1','2','3','4','5','6','s','m'];
        k = 0;
        setOtherController = {};
        for i = 1:1:8
            if(data.dynamics.(sprintf('C%s',j(i))).otherControllerIndex == 1)
                k = k + 1;
                setOtherController{k} = data.dynamics.otherControllers{data.dynamics.(sprintf('C%s',j(i))).otherController};
            end
        end
        
        switch controlArchitecture
            % Settings for corresponding control architectures
            case '4Channel'
                % In the 4Channel configuration all elements stay visible
                if(get(data.handles.designcontrollers,'Value') > 8)
                    set(data.handles.designcontrollers,'Value',1);
                    set(data.handles.(data.designcontrollers),'BackgroundColor',data.colourbutton)
                    data.designcontrollers = 'C1';
                end
                usedControllers = [1 1 1 1 1 1 1 1];
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C2';'Controller C3';'Controller C4';'Controller C5';...
                    'Controller C6';'Controller Cs';'Controller Cm'])
                
                data.dynamics.otherControllers = {'C1','C2','C3','C4','C5','C6','Cs','Cm'};
                
            case 'PERR'
                % In the PERR configuration only C1 Cs Cm and C4 stay visible
                if(get(data.handles.designcontrollers,'Value') > 4)
                    set(data.handles.designcontrollers,'Value',1);
                    set(data.handles.(data.designcontrollers),'BackgroundColor',data.colourbutton)
                    data.designcontrollers = 'C1';
                end
                usedControllers = [1 0 0 1 0 0 1 1];
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C4';'Controller Cs';'Controller Cm'])
                data.dynamics.otherControllers = {'C1','C4','Cs','Cm'};
            case 'SCC'
                % In the SCC configuration only C1 C2 C5 and Cs stay visible
                if(get(data.handles.designcontrollers,'Value') > 4)
                    set(data.handles.designcontrollers,'Value',1);
                    set(data.handles.(data.designcontrollers),'BackgroundColor',data.colourbutton)
                    data.designcontrollers = 'C1';
                end
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C2';'Controller C5';'Controller Cs'])
                usedControllers = [1 1 0 0 1 0 1 0];
                data.dynamics.otherControllers = {'C1','C2','C5','Cs'};
                
            case 'PF'
                % In the PF configuration only C1 C2 and Cs stay visible
                if(get(data.handles.designcontrollers,'Value') > 3)
                    set(data.handles.designcontrollers,'Value',1);
                    set(data.handles.(data.designcontrollers),'BackgroundColor',data.colourbutton)
                    data.designcontrollers = 'C1';
                end
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C2';'Controller Cs'])
                usedControllers = [1 1 0 0 0 0 1 0];
                data.dynamics.otherControllers = {'C1','C2','Cs'};
                %                 controlarchitectures('designcontrollers')
        end
        if k > 0
            for i = 1:1:k
                for j = 1:1:length(data.dynamics.otherControllers)
                    if(strcmp(setOtherController{i},data.dynamics.otherControllers{j}))
                        data.dynamics.(setOtherController{i}).otherController = j;
                    end
                end
            end
        end
        
        % Show controlarchitecure in Lawrence Scheme
        data.designcontrollers = data.dynamics.otherControllers{get(data.handles.designcontrollers,'Value')};
        set(data.handles.(data.designcontrollers),'BackgroundColor',[0 1 0])
         
        data.controlarchitecture = controlArchitecture;
        enableControlBlocks(usedControllers);
        
        set(data.handles.numeratorcontroller,'String',data.dynamics.(data.designcontrollers).Numerator,...
            'Enable','off');
        set(data.handles.denominatorcontroller,'String',data.dynamics.(data.designcontrollers).Denominator,...
            'Enable','off');
        set(data.handles.controllerindex,'String',data.designcontrollers);
    case 'controlarchitecture_load'
        % Here the control architecture that is part of the settingsload
        % is loaded
        switch data.controlarchitecture
            case '4Channel'
                % In the 4Channel configuration all elements stay visible
                usedControllers = [1 1 1 1 1 1 1 1];
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C2';'Controller C3';'Controller C4';'Controller C5';...
                    'Controller C6';'Controller Cs';'Controller Cm'])
                set(data.handles.controlarchitecture,'Value',1);
            case 'PERR'
                % In the PERR configuration only C1 Cs Cm and C4 stay visible
                usedControllers = [1 0 0 1 0 0 1 1];
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C4';'Controller Cs';'Controller Cm'])
                set(data.handles.controlarchitecture,'Value',2);
            case 'SCC'
                % In the SCC configuration only C1 C2 C5 and Cs stay visible
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C2';'Controller C5';'Controller Cs'])
                usedControllers = [1 1 0 0 1 0 1 0];
                set(data.handles.controlarchitecture,'Value',3);
            case 'PF'
                % In the PF configuration only C1 C2 and Cs stay visible
                set(data.handles.designcontrollers,'String',['Controller C1';...
                    'Controller C2';'Controller Cs'])
                set(data.handles.controlarchitecture,'Value',4);
                usedControllers = [1 1 0 0 0 0 1 0];
        end
        
        enableControlBlocks(usedControllers);
    case 'designcontrollers'
        % Evaluate action at the to be designed controller Popup Menu
        
        try
            % Read Out Popup Menu
            designControllers = cellstr(get(hObject,'String'));
            designControllers = designControllers{get(hObject,'Value')};
            
            % Split string from: Controller C* ----> C*
            designControllers = regexp(designControllers,' ','split');
            designControllers = designControllers{2};
        catch
            designControllers = data.designcontrollers;
        end
        % Set colour of to be designed controller to green and reset
        % previous
        set(data.handles.(data.designcontrollers),'BackgroundColor',data.colourbutton);
        set(data.handles.(designControllers),'BackgroundColor',[0 1 0]);
        
        data.designcontrollers = designControllers;
        data.dynamics.Cvar = data.designcontrollers;
        
        set(data.handles.numeratorcontroller,'String',data.dynamics.(data.designcontrollers).Numerator,...
            'Enable','off');
        set(data.handles.denominatorcontroller,'String',data.dynamics.(data.designcontrollers).Denominator,...
            'Enable','off');
        set(data.handles.controllerindex,'String',data.designcontrollers);
        data.ListControllers = {data.designcontrollers};
        if(~isempty(data.Design))
            data.Design = {};
            data.CurController = 1;
            data.Controller = {};
            data.symController = {};
            set(data.handles.controllerindex,'String',{data.designcontrollers},'Value',1);
            
            set(data.handles.showcomponents,'Enable','off','Visible','off','String',{});
            set(data.handles.addcomponent,'Enable','off','Visible','off');
            set(data.handles.removecomponent,'Enable','off','Visible','on');
            set(data.handles.gainedit,'Enable','off','Visible','off');
            set(data.handles.zeroedit,'Enable','off','Visible','off');
            set(data.handles.dampzeroedit,'Enable','off','Visible','off');
            set(data.handles.poleedit,'Enable','off','Visible','off');
            set(data.handles.damppoleedit,'Enable','off','Visible','off');
            set(data.handles.gaintext,'Enable','off','Visible','off');
            set(data.handles.zerotext,'Enable','off','Visible','off');
            set(data.handles.dampzerotext,'Enable','off','Visible','off');
            set(data.handles.poletext,'Enable','off','Visible','off');
            set(data.handles.damppoletext,'Enable','off','Visible','off');
            set(data.handles.unitgain,'Enable','off','Visible','off');
            set(data.handles.unitzero,'Enable','off','Visible','off');
            set(data.handles.unitdampzero,'Enable','off','Visible','off');
            set(data.handles.unitpole,'Enable','off','Visible','off');
            set(data.handles.unitdamppole,'Enable','off','Visible','off');
            designController('initial',hObject,eventdata,handles)
            data.plotController = [1];
        end
        
    case 'designcontrollers_load'
        [~,index] = max(strcmp(data.designcontrollers,data.dynamics.otherControllers));
        set(data.handles.designcontrollers,'Value',index);
        set(data.handles.(data.designcontrollers),'BackgroundColor',[0 1 0]);
end

return

%%
%   function enable ControlBlocks()
%
%   This function enables and disables control blocks if these controllers
%   are not used in the current control architecture.
%
%   Input:          usedControllers: 1x8 vector with 1's that indicate
%                   whether the corresponding controller is used or not.
%                   C = [C1 C2 C3 C4 C5 C6 Cs Cm]
%
%   global data includes the handles to the elements in the GUI
%
%%
function enableControlBlocks(usedControllers)
% Disable controllers that are not used by the controlarchitecture
global data
j = ['1','2','3','4','5','6','s','m'];
for i = 1:1:8
    if(usedControllers(i) == 1)
        set(data.handles.(sprintf('C%s',j(i))),'Enable','on');
    else
        set(data.handles.(sprintf('C%s',j(i))),'Enable','off');
    end
end

return




