%%
%   function createLawrence()
%
%   Creates the Lawrence diagram
%
%%

function createLawrence()
global data

%% Create Lawrence Scheme
% close all;g = figure;

sizeControlBlock = [0.1 0.05];
axes(data.handles.lawrence);
set(data.handles.lawrence,'Units','normalized');

% close figure
% h = figure;
% set(data.handles.lawrence,'Units','normalized','Position',...
%     get(data.handles.lawrence,'Position'));

[r11,r12] = drawcircle(0.15,0.05,1);                                       % Left lower circle for Fh* + ZHXh
[r21,r22] = drawcircle(0.85,0.95,1);                                       % Left lower circle for Fh* + ZHXh
[r31,r32] = drawcircle(0.5,0.125+sizeControlBlock(2)/2,0);                 % Control sum human
[r41,r42] = drawcircle(0.5,0.875-sizeControlBlock(2)/2,0);                 % Control sum slave
[r51,r52] = drawcircle(0.5,0.7,1);                                         % System sum slave
[r61,r62] = drawcircle(0.5,0.3,1);                                         % System sum human

drawarrow(0.05,0.05,0.1-r11,0);                                            % Fh* arrow
drawarrow(0.45,0.05,-0.3+r11,0);                                           % Zh arrow
drawarrow(0.85,0.05,-0.3,0);                                               % Xh to Zh arrow
drawarrow(0.85,0.05,0,0.425);                                              % Xh
drawarrow(0.15,0.95,0.3,0);                                                % Xe to Ze arrow
drawarrow(0.55,0.95,0.3-r11,0);                                            % Ze arrow
drawarrow(0.95,0.95,-0.1+r11,0);                                           % Fe* arrow
drawarrow(0.15,1-0.05,0,-0.425);                                           % Xe
drawarrow(0.85,0.8750-sizeControlBlock(2)/2,-0.09,0);                      % Fe to C5
drawarrow(0.15,0.8750-sizeControlBlock(2)/2,0.09,0);                       % Xe to Cs
drawarrow(0.85,0.125+sizeControlBlock(2)/2,-0.09,0);                       % Xh to Cm
drawarrow(0.15,0.125+sizeControlBlock(2)/2,0.09,0);                        % Fh to C6
drawarrow(0.24+sizeControlBlock(1),0.125+sizeControlBlock(2)/2,...
    .23-sizeControlBlock(1),0);                                            % C6 to Control sum
drawarrow(0.24+sizeControlBlock(1),0.875-sizeControlBlock(2)/2,...
    .23-sizeControlBlock(1),0);                                            % Cs to Control sum
drawarrow(0.76-sizeControlBlock(1),0.875-sizeControlBlock(2)/2,...
    -.23+sizeControlBlock(1),0);                                           % C5 to Control sum
drawarrow(0.76-sizeControlBlock(1),0.125+sizeControlBlock(2)/2,...
    -.23+sizeControlBlock(1),0);                                           % Cm to Control sum
drawarrow(0.15,0.4,0.5-0.15-r41/1.4,-0.25+r42/1.4);                        % C4 to Control sum
drawarrow(0.5666+sizeControlBlock(1)/2,0.4,...
    -0.5666-sizeControlBlock(1)/2+0.5+r41/1.4,-0.25+r42/1.3);              % C2 to Control sum
drawarrow(0.3333+sizeControlBlock(1)/2,0.6,...
    +0.5666+sizeControlBlock(1)/2-0.5-r31/1.4,+0.25-r32/1.3);              % C3 to Control sum
drawarrow(0.85,0.6,-0.85+0.5+r31/1.4,0.25-r32/1.4);                        % C1 to Control sum


drawarrow(0.3333+sizeControlBlock(1)/2,.4,0,.075);                         % to C3
drawarrow(0.5666+sizeControlBlock(1)/2,.6,0,-.075);                        % from C2
drawarrow(0.5+r51,0.3,.23-sizeControlBlock(1)+r51,0);                      % sum to Zm
drawarrow(0.5-r61,0.7,-.23+sizeControlBlock(1)-r61,0);                     % sum to Zs
drawarrow(0.15,0.3,0.5-r51-0.15,0);                                        % Fh to System sum
drawarrow(0.85,0.7,-0.5+r51+0.15,0);                                       % Fe to System sum
drawarrow(0.5,0.15+r42,0,0.3-r52-0.15-r42);                                % Control sum to System sum human
drawarrow(0.5,0.85-r32,0,-0.3+r62+0.15+r42);                               % Control sum to System sum human


line([0.15, 0.15],[0.05+r12, 0.3], 'Color','k');                           % Fh
line([0.85, 0.85],[0.7,0.95-r22], 'Color','k');                            % Fe
line([0.3333+sizeControlBlock(1)/2,0.3333+sizeControlBlock(1)/2],...
    [.6 .525], 'Color','k');                                               % from C3
line([0.5666+sizeControlBlock(1)/2,0.5666+sizeControlBlock(1)/2],...
    [.4 .475],'Color','k');                                                % to C2
line([0.1+sizeControlBlock(1)/2,0.1+sizeControlBlock(1)/2],...
    [.4 .475], 'Color','k');                                               % from C4
line([0.8+sizeControlBlock(1)/2,0.8+sizeControlBlock(1)/2],...
    [.6 .525],'Color','k');                                                % from C1
line([0.15, 0.3333+sizeControlBlock(1)/2],[0.3, 0.4], 'Color','k');        % Schuin Fh
line([0.85, 0.5666+sizeControlBlock(1)/2],[0.7, 0.6], 'Color','k');        % Schuin Fe
line([0.76,0.85],[0.3,0.3],'Color','k');                                   % Zm to Xh
line([0.15,0.24],[0.7,0.7],'Color','k');                                   % Zs to Xe

rectangle('Position', [0.45 0.925 sizeControlBlock])                       % Ze
set(data.handles.Ze,'Position',[.167,.627,.032,.032])
rectangle('Position', [0.45 0.025 sizeControlBlock])                       % Zh
set(data.handles.Zh,'Position',[.167,.080,.032,.032])
rectangle('Position', [0.24 0.7-sizeControlBlock(2)/2 sizeControlBlock])   % Zs
set(data.handles.Zs,'Position',[.101,.475,.032,.032])
rectangle('Position', [0.66 0.3-sizeControlBlock(2)/2 sizeControlBlock])   % Zm
set(data.handles.Zm,'Position',[.2325,.232,.0325,.032])
rectangle('Position', [0.8 0.475 sizeControlBlock])                        % C1
set(data.handles.C1,'Position',[.277,.354,.032,.032])
rectangle('Position', [0.5666 0.475 sizeControlBlock])                     % C2
set(data.handles.C2,'Position',[.2035,.354,.032,.032])
rectangle('Position', [0.3333 0.475 sizeControlBlock])                     % C3
set(data.handles.C3,'Position',[.13,.354,.032,.032])
rectangle('Position', [0.1 0.475 sizeControlBlock])                        % C4
set(data.handles.C4,'Position',[.0565,.354,.0325,.032])
rectangle('Position', [0.66 0.8750-sizeControlBlock(2) sizeControlBlock])  % C5
set(data.handles.C5,'Position',[.2325,.566,.0325,.032])
rectangle('Position', [0.24 0.125 sizeControlBlock])                       % C6
set(data.handles.C6,'Position',[.101,.141,.032,.032])
rectangle('Position', [0.24 0.8750-sizeControlBlock(2) sizeControlBlock])  % Cs
set(data.handles.Cs,'Position',[.101,.566,.032,.032])
rectangle('Position', [0.66 0.125 sizeControlBlock])                       % Cm
set(data.handles.Cm,'Position',[.2325,.141,.0325,.032])



return




function drawarrow(x,y,u,v)

scale = .5/sqrt(u^2+v^2);
h = quiver(x,y,u,v,0,'k');hold on;
adjust_quiver_arrowhead_size(h, scale);

return

function [r1,r2] = drawcircle(x,y,flag)

if(flag)
    r1 = 0.015;
    r2 = 0.01;
else
    r1 = 0.030;
    r2 = 0.02;
end

theta = 0:pi/10:2*pi;
plot(r1*cos(theta)+x,r2*sin(theta)+y,'k');hold on;

return


function adjust_quiver_arrowhead_size(quivergroup_handle, scaling_factor)
% Make quiver arrowheads bigger or smaller.
%
% adjust_quiver_arrowhead_size(quivergroup_handle, scaling_factor)
%
% Example:
%   h = quiver(1:100, 1:100, randn(100, 100), randn(100, 100));
%   adjust_quiver_arrowhead_size(h, 1.5);   % Makes all arrowheads 50% bigger.
%
% Inputs:
%   quivergroup_handle      Handle returned by "quiver" command.
%   scaling_factor          Factor by which to shrink/grow arrowheads.
%
% Output: none

% Kevin J. Delaney
% December 21, 2011
% BMT Scientific Marine Services (www.scimar.com)

if ~exist('quivergroup_handle', 'var')
    help(mfilename);
    return
end

if isempty(quivergroup_handle) || any(~ishandle(quivergroup_handle))
    errordlg('Input "quivergroup_handle" is empty or contains invalid handles.', ...
             mfilename);
    return
end

if length(quivergroup_handle) > 1
    errordlg('Expected "quivergroup_handle" to be a single handle.', mfilename);
    return
end

if ~strcmpi(get(quivergroup_handle, 'Type'), 'hggroup')
    errrodlg('Input "quivergroup_handle" is not of type "hggroup".', mfilename);
    return
end

if ~exist('scaling_factor', 'var') || ...
   isempty(scaling_factor) || ...
   ~isnumeric(scaling_factor)
    errordlg('Input "scaling_factor" is missing, empty or non-numeric.', ...
             mfilename);
    return
end

if length(scaling_factor) > 1
    errordlg('Expected "scaling_factor" to be a scalar.', mfilename);
    return
end

if scaling_factor <= 0
    errordlg('"Scaling_factor" should be > 0.', mfilename);
    return
end

line_handles = get(quivergroup_handle, 'Children');

if isempty(line_handles) || (length(line_handles) < 3) || ...
   ~ishandle(line_handles(2)) || ~strcmpi(get(line_handles(2), 'Type'), 'line')
    errordlg('Unable to adjust arrowheads.', mfilename);
    return
end

arrowhead_line = line_handles(2);

XData = get(arrowhead_line, 'XData');
YData = get(arrowhead_line, 'YData');

if isempty(XData) || isempty(YData)
    return
end

%   Break up XData, YData into triplets separated by NaNs.
first_nan_index = find(~isnan(XData), 1, 'first');
last_nan_index  = find(~isnan(XData), 1, 'last');

for index = first_nan_index : 4 : last_nan_index
    these_indices = index + (0:2);
    
    if these_indices(end) > length(XData)
        break
    end
    
    x_triplet = XData(these_indices);
    y_triplet = YData(these_indices);
    
    if any(isnan(x_triplet)) || any(isnan(y_triplet))
        continue
    end
    
    %   First pair.
    delta_x = diff(x_triplet(1:2));
    delta_y = diff(y_triplet(1:2));
    x_triplet(1) = x_triplet(2) - (delta_x * scaling_factor);
    y_triplet(1) = y_triplet(2) - (delta_y * scaling_factor);
        
    %   Second pair.
    delta_x = diff(x_triplet(2:3));
    delta_y = diff(y_triplet(2:3));
    x_triplet(3) = x_triplet(2) + (delta_x * scaling_factor);
    y_triplet(3) = y_triplet(2) + (delta_y * scaling_factor);
    
    XData(these_indices) = x_triplet;
    YData(these_indices) = y_triplet;
end

set(arrowhead_line, 'XData', XData, 'YData', YData);