%%
%   function definedynamics
%
%   This function is called when pushing a button in the Lawrence scheme
%   and the user wants to define certain dynamics or controllers. When the
%   pushbutton is pressed, a popupscreen is created that allows the user to
%   choose how the dynamics are defined (as impedance or tf)
%
%   Input:          action: String that indicates which push button in pressed
%                   hObject,eventdata,handles are standard inputs and are not
%                   used
%
%%

function definedynamics(action,hObject,eventdata,handles)
%
global data

switch action
    case 'Zm'
        Name = 'Master dynamics';
        Abbreviation = 'Zm';
    case 'Zs'
        Name = 'Slave Dynamics';
        Abbreviation = 'Zs';
    case 'C1'
        Name = 'Controller C1';
        Abbreviation = 'C1';
    case 'C2'
        Name = 'Controller C2';
        Abbreviation = 'C2';
    case 'C3'
        Name = 'Controller C3';
        Abbreviation = 'C3';
    case 'C4'
        Name = 'Controller C4';
        Abbreviation = 'C4';
    case 'C5'
        Name = 'Controller C5';
        Abbreviation = 'C5';
    case 'C6'
        Name = 'Controller C6';
        Abbreviation = 'C6';
    case 'Cs'
        Name = 'Controller Cs';
        Abbreviation = 'Cs';
    case 'Cm'
        Name = 'Controller Cm';
        Abbreviation = 'Cm';
    case 'Ze'
        Name = 'Environment Dynamics';
        Abbreviation = 'Ze';
    case 'Zh'
        Name = 'Human Dynamics';
        Abbreviation = 'Zh';
    case 'definitionCheck'
        % This case checks whether enough dynamics are defined for
        % evaluation
        %         items = data.dynamics.elements;
        %         n = length(data.dynamics.elements);
        %
        %         counter = 0;
        %
        %         % It is not necessary for the environment dynamics and human
        %         % dynamics to be defined as non zero. At least one controller
        %         % should be defined as nonzero and both master and slave dynamics
        %         % should be defined as nonzero.
        %         for i = 3:1:n
        %             if(str2num(data.dynamics.(items{i}).Numerator) == 0)
        %                 switch items{i}
        %                     case {'Zs','Zm'}
        %                         counter = 8;
        %                 end
        %                 counter = counter + 1;
        %             end
        %         end
        %         if(counter >= 8)
        %             errorHandling('Not enough dynamics defined');
        %             return
        %         end
        
        return
end

data.dynamics.temp.Name = Name;
data.dynamics.temp.Abbr = Abbreviation;

createDeclarationPopUpMenu();

% In the text screen right below in the GUI Every action should be defined
set(data.handles.defineElement,'string',['Defining ', data.dynamics.temp.Name]); drawnow;

return

%%
%   function createDeclarationPopUpMenu()

%   The function creates a Declaration Pop-up with a title
%   corresponding to what is declared. This Pop-up shows a Cancel button
%   and a Define button, for cancelling and saving the input respectively.
%
%
%
%%
function createDeclarationPopUpMenu(varargin)
global data

Name = data.dynamics.temp.Name;
Abbreviation = data.dynamics.temp.Abbr;

dyn = data.dynamics.(Abbreviation);
dyntemp = data.dynamics.temp.(Abbreviation);

% Declaration Pop-up in the middle of the screen
h = figure;
data.dynamics.temp.h = h;
hDesiredSize = [520, 160];
set(h,'Resize','off','MenuBar','none','ToolBar','none','NumberTitle','off','Name',...
    ['Defining ', Name],'Position',centerPopUp(hDesiredSize),...
    'Units','normalized','Color',[.9 .9 .9]);%,'WindowStyle','modal');

% Declaration Cancel button
hcancel = uicontrol(h,'Style','pushbutton','String','Cancel','Units','normalized',...
    'Position',[.25 .06 .2 .2],'Backgroundcolor',data.colourbutton,'FontSize',[12],...
    'Callback','global data; closePopUp(data.dynamics.temp.h); clear data;');

% Declaration Define button
hdefine = uicontrol(h,'Style','pushbutton','String','Define','Units','normalized',...
    'Position',[.55 .06 .2 .2],'Backgroundcolor',data.colourbutton,'FontSize',[12],...
    'Callback',@saveDeclaration);

listbox = uicontrol(h,'Style','popupmenu','Backgroundcolor',[1 1 1],'FontSize',[10],...
    'Units','normalized','Value',1,'String',['Temp']);

dyntemp.NumeratorString = uicontrol(h,'Style','text','String','Numerator ',...
    'Units','normalized','Position',[.37 .65 .3 .3],'Backgroundcolor',...
    [.9 .9 .9]);
dyntemp.DenominatorString = uicontrol(h,'Style','text','String','Denominator',...
    'Units','normalized','Position',[.37 .35 .3 .3],'Backgroundcolor',...
    [.9 .9 .9]);


dyntemp.NumeratorSort = uicontrol(h,'Style','text','String',dyn.SortInput,...
    'Units','normalized','Position',[.60 .65 .1 .3],'Backgroundcolor',...
    [.9 .9 .9]);
dyntemp.DenominatorSort = uicontrol(h,'Style','text','String',dyn.SortInput,...
    'Units','normalized','Position',[.60 .35 .1 .3],'Backgroundcolor',...
    [.9 .9 .9]);

dyntemp.Numerator = uicontrol(h,'Style','edit',...
    'String',[num2str(dyn.Numerator)],...
    'Units','normalized','UserData',dyn.Numerator,...
    'Backgroundcolor',[1 1 1],'Position',[.35 .7 .35 .15]);
dyntemp.Denominator = uicontrol(h,'Style','edit',...
    'String',num2str(dyn.Denominator),...
    'UserData',dyn.Denominator,...
    'Units','normalized','Backgroundcolor',[1 1 1],'Position',[.35 .4 .35 .15]);


switch Abbreviation
    case {'Zm','Zs'}
        % Declaration Pop-up Menu button for what format the input is
        
        set(listbox,'String',{'Transfer function','Impedance'},...
            'Position',[.05 .25 .25 .3],'Value',...
            dyn.ZorTf,'Callback',@changeText);
        
        uicontrol(h,'Style','text','String','Dynamics with respect to:',...
            'Units','normalized','Position',[.025 .65 .3 .3],'Backgroundcolor',...
            [.9 .9 .9]);
        uicontrol(h,'Style','text','String','Defined as a',...
            'Units','normalized','Position',[.013 .55 .2 .1],'Backgroundcolor',...
            [.9 .9 .9]);
        indXorV = uicontrol(h,'Style','popupmenu','String',{['Position (s',...
            Abbreviation,')'],['Velocity (',Abbreviation,')']},...
            'Units','normalized','Position',[.05 .55 .25 .3],'Backgroundcolor',[1 1 1],...
            'Value',dyn.XorV,'FontSize',[10],'Callback',@changeText);
        dyntemp.XorV = indXorV;
        
        dyntemp.listbox = listbox;
        
        uicontrol(h,'Style','text','String','Load','Units','normalized',...
            'Position',[.77 .8 .15 .15],'Backgroundcolor',[.9 .9 .9]);
        dyntemp.LoadFromWorkspace = uicontrol(h,'Style','pushbutton','String',...
            'From Workspace','units','normalized','Position',[.72 .7 .25 .15],...
            'Callback',@loadfromworkspace);
        dyntemp.LoadFromFile = uicontrol(h,'Style','pushbutton','String',...
            'From File','units','normalized','Position',[.72 .55 .25 .15],...
            'Callback',@loadfromfile);
        
    case {'C1','C2','C3','C4','C5','C6','Cs','Cm'}
        % Declaration of Controller
        set(listbox,'String',['Controller ',Abbreviation],...
            'Position',[.05 .65 .2 .25]);
        
        % Radiobutton to indicate whether controller is function of other
        % controller
        Index = uicontrol(h,'Style','radiobutton','Units','normalized',...
            'Position',[.05 .55 .3 .1],'String','Function of other controller',...
            'Backgroundcolor',[.9 .9 .9],'Value',...
            dyn.otherControllerIndex,'Callback',...
            @funcOtherController);
        
        dyntemp.listbox = listbox;
        dyntemp.otherControllerIndex = get(Index,'Value');% 1 or 0
        
        k = 1;
        for i = 1:1:length(data.dynamics.otherControllers)
            if(~strcmp(data.dynamics.otherControllers{i},Abbreviation))
                listCont{k} = data.dynamics.otherControllers{i};
                k = k + 1;
            else
                if(k < dyn.otherController)
                    otherControllerIndex = dyn.otherController-1;
                else
                    otherControllerIndex = dyn.otherController;
                end
            end
        end
        
        
        dyntemp.otherController =...
            uicontrol(h,'Style','popupmenu','String',listCont,...
            'Units','normalized','Value',otherControllerIndex,...
            'Backgroundcolor',[1 1 1],'Position',[.08 .45 .2 .1],'Enable','off');
        if(dyn.otherControllerIndex == 1)
            set(dyntemp.otherController,'Enable','on');
        end
        dyntemp.otherControllerString = uicontrol(h,'Style','edit','String',...
            dyn.otherControllerString,'Units','normalized','UserData',...
            dyn.otherControllerString,'Backgroundcolor',[1 1 1],'Position',...
            [.35 .7 .35 .15],'Visible','off');
        
        if(dyntemp.otherControllerIndex == 1)
            set(dyntemp.Numerator,'Visible','off')
            set(dyntemp.Denominator,'Visible','off')
            set(dyntemp.DenominatorString,'Visible','off')
            set(dyntemp.NumeratorString,'Visible','off')
            set(dyntemp.otherControllerString,'Visible','on')
            set(dyntemp.NumeratorSort,'Visible','off')
            set(dyntemp.DenominatorSort,'Visible','off')
        end
        
        dyntemp.LoadFromWorkspace = uicontrol(h,'Style','pushbutton','String',...
            'From Workspace','units','normalized','Position',[.72 .7 .25 .15],...
            'Callback',@loadfromworkspace);
        dyntemp.LoadFromFile = uicontrol(h,'Style','pushbutton','String',...
            'From File','units','normalized','Position',[.72 .55 .25 .15],...
            'Callback',@loadfromfile);        
        
    case {'Zh','Ze'}
        
        % Declaration of human or environment dynamics
        set(listbox,'String','Impedance','Position',[.05 .25 .25 .3],'Value',...
            dyn.ZorTf,'Callback',@changeText);
        
        uicontrol(h,'Style','text','String','Dynamics with respect to:',...
            'Units','normalized','Position',[.025 .65 .3 .3],'Backgroundcolor',...
            [.9 .9 .9]);
        uicontrol(h,'Style','text','String','Defined as a',...
            'Units','normalized','Position',[.013 .55 .2 .1],'Backgroundcolor',...
            [.9 .9 .9]);
        indXorV = uicontrol(h,'Style','popupmenu','String',{['Position (s',...
            Abbreviation,')'],['Velocity (',Abbreviation,')']},...
            'Units','normalized','Position',[.05 .55 .25 .3],'Backgroundcolor',[1 1 1],...
            'Value',dyn.XorV,'FontSize',[10],'Callback',@changeText);
        set(data.handles.Ze,'Enable','on');
        dyntemp.XorV = indXorV;
        dyntemp.listbox = listbox;
        
        dyntemp.LoadFromWorkspace = uicontrol(h,'Style','pushbutton','String',...
            'From Workspace','units','normalized','Position',[.72 .7 .25 .15],...
            'Callback',@loadfromworkspace);
        dyntemp.LoadFromFile = uicontrol(h,'Style','pushbutton','String',...
            'From File','units','normalized','Position',[.72 .55 .25 .15],...
            'Callback',@loadfromfile);
        
end

data.dynamics.(Abbreviation) = dyn;
data.dynamics.temp.(Abbreviation) = dyntemp;

return


function funcOtherController(varargin)
global data

Abbreviation = (data.dynamics.temp.Abbr);
dyntemp = data.dynamics.temp.(Abbreviation);

if(dyntemp.otherControllerIndex == 0)
    set(dyntemp.otherController,'Enable','on');
    dyntemp.otherControllerIndex = 1;
    set(dyntemp.Numerator,'Visible','off');
    set(dyntemp.Denominator,'Visible','off');
    set(dyntemp.NumeratorString,'Visible','off');
    set(dyntemp.DenominatorString,'Visible','off');
    set(dyntemp.otherControllerString,'Visible','on');
    set(dyntemp.NumeratorSort,'Visible','off');
    set(dyntemp.DenominatorSort,'Visible','off');
else
    set(dyntemp.otherController,'Enable','off');
    dyntemp.otherControllerIndex = 0;
    set(dyntemp.Numerator,'Visible','on');
    set(dyntemp.NumeratorString,'Visible','on');
    set(dyntemp.Denominator,'Visible','on');
    set(dyntemp.DenominatorString,'Visible','on');
    set(dyntemp.otherControllerString,'Visible','off');
    set(dyntemp.NumeratorSort,'Visible','on');
    set(dyntemp.DenominatorSort,'Visible','on');
end

data.dynamics.temp.(Abbreviation) = dyntemp;
return

function saveDeclaration(varargin)
global data
Abbreviation = data.dynamics.temp.Abbr;
dyn = data.dynamics.(Abbreviation);
dyntemp = data.dynamics.temp.(Abbreviation);

if(length(get(dyntemp.Numerator,'String')) == 0)
    set(dyntemp.Numerator,'String',0);
end
syms a b c d e f g h i j k l m n o p q r s t u v w x y z
if(length(get(dyntemp.Denominator,'String')) == 0 || eval(get(dyntemp.Denominator,'String')) == 0)
    errorHandling('Input Denominator required. (cannot divide by 0)')
    return
end
dyn.Numerator = get(dyntemp.Numerator,'String');
dyn.Denominator = get(dyntemp.Denominator,'String');
dyn.SortInput = get(dyntemp.NumeratorSort,'String');

dyn.ZorTf =...
    get(dyntemp.listbox,'Value');

switch(Abbreviation)
    case 'Zs'
        dyn.XorV =...
            get(dyntemp.XorV,'Value');
        if( dyn.XorV == 1 )
            % Position
            
            set(data.handles.XeVe,'String','Xe');
            if (dyn.ZorTf == 1)
                % Transfer function
                set(data.handles.(Abbreviation),'String',...
                    ['1/(s',Abbreviation,')']);
            else
                set(data.handles.(Abbreviation),'String',...
                    Abbreviation);
            end
            
        else
            set(data.handles.XeVe,'String','Ve');
            if (dyn.ZorTf == 1)
                % Transfer function
                set(data.handles.(Abbreviation),'String',...
                    ['1/(',Abbreviation,')']);
            else
                set(data.handles.(Abbreviation),'String',...
                    Abbreviation);
            end
        end
        
    case 'Zm'
        dyn.XorV =...
            get(dyntemp.XorV,'Value');
        if( dyn.XorV == 1 )
            % Position
            set(data.handles.XhVh,'String','Xh');
            if (dyn.ZorTf == 1)
                % Transfer function
                set(data.handles.(Abbreviation),'String',...
                    ['1/(s',Abbreviation,')']);
            else
                set(data.handles.(Abbreviation),'String',...
                    Abbreviation);
            end
            
        else
            set(data.handles.XhVh,'String','Vh');
            if (dyn.ZorTf == 1)
                % Transfer function
                set(data.handles.(Abbreviation),'String',...
                    ['1/(',Abbreviation,')']);
            else
                set(data.handles.(Abbreviation),'String',...
                    Abbreviation);
            end
        end
    case {'Ze','Zh'}
        dyn.XorV =...
            get(dyntemp.XorV,'Value');
        
        if( dyn.XorV == 1 )
            set(data.handles.(Abbreviation),'String',...
                ['s',Abbreviation]);
        else
            set(data.handles.(Abbreviation),'String',...
                Abbreviation);
        end
        
    case {'C1','C2','C3','C4','C5','C6','Cs','Cm'}
        set(data.handles.(Abbreviation),'String',Abbreviation);
        dyn.otherControllerIndex = dyntemp.otherControllerIndex;
        
        if(dyn.otherControllerIndex == 1)
            % Controller is function of other controller
            
            posInp = get(dyntemp.otherController,'String');% String of possible inputs
            Input = posInp{get(dyntemp.otherController,'Value')};
            if(strcmp(Input,Abbreviation))
                errorHandling([Input,' is not another controller']);
                return
            end
            
            [~,dyn.otherController] = max(strcmp(Input,data.dynamics.otherControllers));
            dyn.otherControllerString = get(dyntemp.otherControllerString,'String');
            stringController = strfind(dyn.otherControllerString,Input);
            %             n = length(stringController);
            %             p = length(Input);
            
            if(length(stringController) == 0)
                errorHandling(['Defined controller ',Abbreviation,' is not a function of ',Input])
                return

            else
                dyn.Numerator = strrep(dyn.otherControllerString,Input,...
                    strcat('(',data.dynamics.(Input).Numerator,')','/',...
                    '(',data.dynamics.(Input).Denominator,')'));
                dyn.Denominator = num2str(1);
            end
            
            for i = 1:1:8
                % Check if other controllers are a function of this
                % controller
                if(~strcmp(data.dynamics.elements{i+4},Abbreviation))
                    element = data.dynamics.(data.dynamics.elements{i+4});
                    if(strcmp(data.dynamics.otherControllers{element.otherController},Abbreviation) &&...
                            data.dynamics.(data.dynamics.elements{i+4}).otherControllerIndex)
                        strCon = strfind(element.otherControllerString,...
                            data.dynamics.otherControllers{element.otherController});
                        nE = length(strCon);
                        p = length(data.dynamics.otherControllers{element.otherController});
                        if(strCon(1) ~= 1)
                            for j = 1:1:nE
                                element.Numerator = [element.otherControllerString(1:strCon(j)-1),'(',...
                                    dyn.Numerator,')',...
                                    element.otherControllerString(strCon(j)+p:end)];
                                element.Denominator = num2str(1);
                            end
                        else
                            element.Numerator = dyn.Numerator;
                            element.Denominator = num2str(1);
                        end
                    end
                    data.dynamics.(data.dynamics.elements{i+4}) = element;
                end
                
            end
            
            
        else
            if(max(strcmp(dyn.Numerator,{'C1','C2','C3','C4','C5','C6','Cs','Cm'})))
                errorHandling('Controller is function of other controller')
                return
            end
        end
        if(strcmp(Abbreviation,data.designcontrollers))
            set(data.handles.numeratorcontroller,'String',dyn.Numerator);
            set(data.handles.denominatorcontroller,'String',dyn.Denominator);
        end
end

closePopUp(data.dynamics.temp.h);
set(data.handles.defineElement,'String',' '); drawnow;

data.dynamics.(Abbreviation) = dyn;
data.dynamics.temp.(Abbreviation) = dyntemp;

return

function changeText(varargin)
global data

Abbreviation = data.dynamics.temp.Abbr;
dyntemp = data.dynamics.temp.(Abbreviation);

switch Abbreviation
    case {'Zs','Zm'}
        if(get(dyntemp.XorV,'Value') == 1)
            if(get(dyntemp.listbox,'Value') == 1)
                set(dyntemp.NumeratorSort,'String',strcat('1/s',Abbreviation));
                set(dyntemp.DenominatorSort,'String',strcat('1/s',Abbreviation));
            else
                set(dyntemp.NumeratorSort,'String',strcat('s',Abbreviation));
                set(dyntemp.DenominatorSort,'String',strcat('s',Abbreviation));
            end
        else
            if(get(dyntemp.listbox,'Value') == 1)
                set(dyntemp.NumeratorSort,'String',strcat('1/',Abbreviation));
                set(dyntemp.DenominatorSort,'String',strcat('1/',Abbreviation));
            else
                set(dyntemp.NumeratorSort,'String',strcat(Abbreviation));
                set(dyntemp.DenominatorSort,'String',strcat(Abbreviation));
            end
        end
    case {'Ze','Zh'}
        if(get(dyntemp.XorV,'Value') == 1)
            
            set(dyntemp.NumeratorSort,'String',strcat('s',Abbreviation));
            set(dyntemp.DenominatorSort,'String',strcat('s',Abbreviation));
        else
            
            set(dyntemp.NumeratorSort,'String',strcat(Abbreviation));
            set(dyntemp.DenominatorSort,'String',strcat(Abbreviation));
        end
        
end
return
