


%%
%       function designController
%
%       This function is used for the design of controllers using
%       loop-shaping. The components can be chosen and altered. This
%       function contains all elements of the Control Design panel.
%
%%
function designController(action,hObject,eventdata,handles)
global data

switch action
    case 'initial'
        num = data.dynamics.(data.designcontrollers).Numerator;
        den = data.dynamics.(data.designcontrollers).Denominator;
        
        % Matrix multiplications need a '.'
        num = strrep(num,'*','.*');
        den = strrep(den, '*', '.*');
        num = strrep(num,'/','./');
        den = strrep(den, '/', './');
        num = strrep(num,'^','.^');
        den = strrep(den, '^', '.^');
        
        syms s;
        data.symController{1} = eval(num)/eval(den);
        
        if(data.mapped == 1)
            if(data.plot.unitsFreq.Value == 1)
                s = 1j*2*pi*data.parameters.Freq;
            else
                s = 1j*data.parameters.Freq;
            end
        end
        data.Controller{1} = eval(num)./ eval(den);
        data.CurController = 1;
        data.Design{data.CurController}.vecElements{data.CurController} = [1 1 1 1 1 1 1 1 1 1 1 1 1 1 1];
        data.Design{data.CurController}.Name = data.designcontrollers;
        data.Design{data.CurController}.Components = {};
        
    case 'regular'
        
        % Create a new controller design
        data.DesignNaming = length(data.Design);
        
        Title = data.ListControllers{1};
        if(length(data.ListControllers)>1)
            for i = 2:1:length(data.ListControllers)
                Title = [Title,' = ',data.ListControllers{i}];
            end
        end
        
        h = figure;
        data.dynamics.temp.h = h;
        hDesiredSize = [320, 200];
        set(h,'Resize','off','MenuBar','none','ToolBar','none','NumberTitle','off','Name',...
            ['Design Controller ',Title],'Position',centerPopUp(hDesiredSize),...
            'Units','normalized','Color',[.9 .9 .9],'WindowStyle','modal');
        
        % Declaration Cancel button
        hcancel = uicontrol(h,'Style','pushbutton','String','Cancel','Units','normalized',...
            'Position',[.4 .06 .2 .15],'Backgroundcolor',data.colourbutton,'FontSize',[11],...
            'Callback','global data; closePopUp(data.dynamics.temp.h); otherButtons(''enable'');clear data;');
        
        uicontrol(h,'Style','pushbutton','String','Complete New Controller',...
            'Units','normalized','Position',[.1 .8 .8 .15],'Backgroundcolor',data.colourbutton,...
            'Value',0,'FontSize',[10],'Callback',@createOwnController);
        uicontrol(h,'Style','pushbutton','String','Alter Current Controller',...
            'Units','normalized','Position',[.1 .6 .8 .15],'Backgroundcolor',data.colourbutton,...
            'Value',0,'FontSize',[10],'Callback',@createComposedController);
        uicontrol(h,'Style','pushbutton','String','Load Existing Controller',...
            'Units','normalized','Position',[.1 .4 .8 .15],'Backgroundcolor',data.colourbutton,...
            'Value',0,'FontSize',[10],'Callback',@loadExistingController);
        
    case 'choosecontroller'
        % Read Out Popup Menu
        
        % navigate through designs.
        
        data.CurController = get(data.handles.controllerindex,'Value');
        
        if(isempty(data.Design{data.CurController}.Components))
            set(data.handles.showcomponents,'Enable','off','Visible','off');
            set(data.handles.addcomponent,'Enable','off','Visible','off');
            for i = 1:1:length(data.strElements)
                set(data.handles.(data.strElements{i}),'Visible','off','Enable','off');
            end
            if(data.CurController == 1)
                set(data.handles.numeratorcontroller,'Enable','off');
                set(data.handles.denominatorcontroller,'Enable','off');
            else
                set(data.handles.numeratorcontroller,'Enable','on');
                set(data.handles.denominatorcontroller,'Enable','on');
            end
            num = data.Design{data.CurController}.num;
            den = data.Design{data.CurController}.den;            
            
        else
            set(data.handles.denominatorcontroller,'Enable','off');
            set(data.handles.numeratorcontroller,'Enable','off');
            if(data.Design{data.CurController}.Components{1}(1) == 1)
                % gain
                num = strcat('(',data.Design{data.CurController}.num,')*(',...
                    data.Design{data.CurController}.Components{2},')');
                den = data.Design{data.CurController}.den;
                set(data.handles.gainedit,'String',data.Design{data.CurController}.gainValue{1});
            else
                % other components
                num = data.Design{data.CurController}.ComponentsNum{1};
                den = data.Design{data.CurController}.ComponentsDen{1};
                set(data.handles.zeroedit,'String',data.Design{data.CurController}.zeroValue{1});
                set(data.handles.poleedit,'String',data.Design{data.CurController}.poleValue{1});
                set(data.handles.dampzeroedit,'String',data.Design{data.CurController}.dampzeroValue{1});
                set(data.handles.damppoleedit,'String',data.Design{data.CurController}.damppoleValue{1});
            end
            listOfAllComp = get(data.handles.addcomponent,'String');
            for i = 1:1:length(data.Design{data.CurController}.Components{1})
                listOfComp{i} = listOfAllComp{data.Design{data.CurController}.Components{1}(i)+1};
            end
            
            set(data.handles.showcomponents,'Enable','on','Visible','on','String',...
                listOfComp,'Value',1);
            set(data.handles.addcomponent,'Enable','on','Visible','on');
            
            setEdit(1);
            
        end
        
        if(data.plotController(data.CurController) == 1)
            set(data.handles.plotcontroller,'Value',1);
        else
            set(data.handles.plotcontroller,'Value',0);
        end
        
        evaluateStability('final');
        
        set(data.handles.numeratorcontroller,'String',num);
        set(data.handles.denominatorcontroller,'String',den);
        
    case 'plotcontroller'
        % Only a controller will be plotted or removed
        
        if(get(hObject,'Value') == 1)
            data.plotController(data.CurController) = 1;
            plotController();
        else
            data.plotController(data.CurController) = 0;
            set(data.plot.Magcontroller{data.CurController},'Visible','off')
            set(data.plot.Phasecontroller{data.CurController},'Visible','off')
            set(data.plot.Pointcontroller{data.CurController},'Visible','off')
        end
        
    case 'changedefinedtransferfunction'
        % The transfer function in the numerator and denominator panel will
        % be changed.
        
        data.Design{data.CurController}.num = get(data.handles.numeratorcontroller,'String');
        data.Design{data.CurController}.den = get(data.handles.denominatorcontroller,'String');       
        
        delete(data.plot.Magcontroller{data.CurController});
        data.plot.Magcontroller{data.CurController} = {};
        delete(data.plot.Phasecontroller{data.CurController})
        data.plot.Phasecontroller{data.CurController} = {};
        delete(data.plot.Pointcontroller{data.CurController})
        data.plot.Pointcontroller{data.CurController} = {};
        
        set(data.plot.Magcontroller{data.CurController},'Visible','off')
        set(data.plot.Phasecontroller{data.CurController},'Visible','off')
        set(data.plot.Pointcontroller{data.CurController},'Visible','off')
        
        data.plot.Magcontroller{data.CurController} = {};
        data.plot.Phasecontroller{data.CurController} = {};
        data.plot.Pointcontroller{data.CurController} = {};
        
        if(isempty(data.Design{data.CurController}.Components))
            evaluateDefinedController('on')
        else
            evaluateDefinedController('off')
        end
        
    case 'addcomponent'
        % Add a loop-shaping component
        
        compString = cellstr(get(hObject,'String'));
        component = compString{get(hObject,'Value')};
        
        if(~strcmp(component,'add'))
            addcomponent(component);
        end
        
        set(data.handles.addcomponent,'Value',1);
        
    case 'remove'
        % Remove a loop-shaping component
        removeString = cellstr(get(hObject,'String'));
        remove = removeString{get(hObject,'Value')};
        
        removeComponent(remove);
        
    case 'gain'
        % Change the gain in the Control Design panel
        
        input = get(data.handles.gainedit,'String');
        if(strfind(input,'.') == 1)
            input = strcat('0',input);
        end
        if(strfind(input,'.') == length(input))
            input = strcat(input,'0');
        end
        set(data.handles.numeratorcontroller,'String',...
            strcat('(',input,')*(',data.Design{data.CurController}.num,')'));
        set(data.handles.denominatorcontroller,'String',...
            data.Design{data.CurController}.den);
        
        set(data.plot.Magcontroller{data.CurController},'Visible','off');
        set(data.plot.Phasecontroller{data.CurController},'Visible','off');
        set(data.plot.Pointcontroller{data.CurController},'Visible','off');
        
        data.plot.Magcontroller{data.CurController} = {};
        data.plot.Phasecontroller{data.CurController} = {};
        data.plot.Pointcontroller{data.CurController} = {};
        
        for i = 1:1:length(data.Design{data.CurController}.Components{1})
            if(data.Design{data.CurController}.Components{1}(i) == 1)
                data.Design{data.CurController}.Components{i+1} = input;
            end
        end
        
        evaluateDefinedController('off');
        
        data.Design{data.CurController}.gainValue{get(data.handles.showcomponents,'Value')} =...
            get(data.handles.gainedit,'String');
        data.Design{data.CurController}.ComponentsNum{get(data.handles.showcomponents,'Value')} =...
            strcat('(',get(data.handles.gainedit,'String'),')*(',data.Design{data.CurController}.num,')');
        
    case {'zero','pole','dampzero','damppole'}
        % Change the other parameters in the Control Design panel

        data.Design{data.CurController}.zeroValue{get(data.handles.showcomponents,'Value')} =...
            get(data.handles.zeroedit,'String');
        data.Design{data.CurController}.poleValue{get(data.handles.showcomponents,'Value')} =...
            get(data.handles.poleedit,'String');
        data.Design{data.CurController}.dampzeroValue{get(data.handles.showcomponents,'Value')} =...
            get(data.handles.dampzeroedit,'String');
        data.Design{data.CurController}.damppoleValue{get(data.handles.showcomponents,'Value')} =...
            get(data.handles.damppoleedit,'String');
        
        set(data.plot.Magcontroller{data.CurController},'Visible','off')
        set(data.plot.Phasecontroller{data.CurController},'Visible','off')
        set(data.plot.Pointcontroller{data.CurController},'Visible','off')
        
        data.plot.Magcontroller{data.CurController} = {};
        data.plot.Phasecontroller{data.CurController} = {};
        data.plot.Pointcontroller{data.CurController} = {};
        
        [f1 f2 b1 b2 P D] = getEdit(get(data.handles.showcomponents,'Value'));
        
        num = data.Design{data.CurController}.ComponentsNum{get(data.handles.showcomponents,'Value')};
        den = data.Design{data.CurController}.ComponentsDen{get(data.handles.showcomponents,'Value')};
        
        repNum = subsComp(num,f1, f2, b1, b2, P, D);
        repDen = subsComp(den,f1, f2, b1, b2, P, D);
        
        set(data.handles.numeratorcontroller,'String',repNum);
        set(data.handles.denominatorcontroller,'String',repDen);
        
        evaluateDefinedController('off')
        
    case 'showcomponents'
        % Choose in list with added components
        
        allComponents = get(data.handles.showcomponents,'String');
        ComponentIndex = get(data.handles.showcomponents,'Value');
        
        setEdit(ComponentIndex)
        set(data.handles.gainedit,'String',data.Design{data.CurController}.gainValue{ComponentIndex});
        set(data.handles.zeroedit,'String',data.Design{data.CurController}.zeroValue{ComponentIndex});
        set(data.handles.poleedit,'String',data.Design{data.CurController}.poleValue{ComponentIndex});
        set(data.handles.dampzeroedit,'String',data.Design{data.CurController}.dampzeroValue{ComponentIndex});
        set(data.handles.damppoleedit,'String',data.Design{data.CurController}.damppoleValue{ComponentIndex});
        
        if(data.Design{data.CurController}.Components{1}(ComponentIndex) == 7)
            set(data.handles.zerotext,'String','P');
            set(data.handles.poletext,'String','D');
        else
            set(data.handles.zerotext,'String','zero');
            set(data.handles.poletext,'String','pole');
        end
        
        if(data.Design{data.CurController}.Components{1}(ComponentIndex) == 1)
            set(data.handles.numeratorcontroller,'String',strcat('(',...
                data.Design{data.CurController}.gainValue{ComponentIndex},')*(',data.Design{data.CurController}.num,')'));
            set(data.handles.denominatorcontroller,'String',data.Design{data.CurController}.den);
        else
            [f1 f2 b1 b2 P D] = getEdit(ComponentIndex);
        
        num = data.Design{data.CurController}.ComponentsNum{ComponentIndex};
        den = data.Design{data.CurController}.ComponentsDen{ComponentIndex};
        repNum = subsComp(num, f1,f2, b1, b2, P ,D);
        repDen = subsComp(den, f1,f2, b1, b2, P ,D);
            set(data.handles.numeratorcontroller,'String',repNum);
            set(data.handles.denominatorcontroller,'String',repDen);
        end
        
        
end

return



%%
%       function createComposedController
%       
%       This function takes the original controller and allows the user to
%       add other components like lead filters or gains etc.
%
%%
function createComposedController(varargin)
global data

closePopUp(data.dynamics.temp.h);

Title = data.ListControllers{1};
if(length(data.ListControllers)>1)
    for i = 2:1:length(data.ListControllers)
        Title = [Title,' = ',data.ListControllers{i}];
    end
end

h = figure;
data.dynamics.temp.h = h;
hDesiredSize = [220, 160];
set(h,'Resize','off','MenuBar','none','ToolBar','none','NumberTitle','off','Name',...
    ['Design Controller ',Title],'Position',centerPopUp(hDesiredSize),...
    'Units','normalized','Color',[.9 .9 .9],'WindowStyle','modal');

% Declaration Cancel button
hcancel = uicontrol(h,'Style','pushbutton','String','Cancel','Units','normalized',...
    'Position',[.20 .06 .3 .2],'Backgroundcolor',data.colourbutton,'FontSize',[9],...
    'Callback','global data; closePopUp(data.dynamics.temp.h); clear data;');

% Declaration Define button
hdefine = uicontrol(h,'Style','pushbutton','String','Define','Units','normalized',...
    'Position',[.55 .06 .3 .2],'Backgroundcolor',data.colourbutton,'FontSize',[9],...
    'Callback',@defineAlteringController);

uicontrol(h,'Style','text','String','Name','Units','normalized',...
    'Position',[.4 .70 .2 .1],'Backgroundcolor',[.9 .9 .9]);
data.dynamics.temp.addcontroller.Name = uicontrol(h,'Style','edit',...
    'String',[data.ListControllers{1},'_',num2str(data.DesignNaming)],...
    'Units','normalized','Backgroundcolor',[1 1 1],'Position',[.25 .45 .5 .2]);

data.dynamics.temp.addcontroller.Numerator = data.Design{1}.num;
data.dynamics.temp.addcontroller.Denominator = data.Design{1}.den;

return

%%
%       function defineAlteringController
%       
%       This function defines and saves the original controller with the
%       additional components etc.
%
%%
function defineAlteringController(varargin)
global data

set(data.handles.numeratorcontroller,'Enable','off')
set(data.handles.denominatorcontroller,'Enable','off')
set(data.handles.showcomponents,'Enable','on','Visible','on')
set(data.handles.addcomponent,'Enable','on','Visible','on')
set(data.handles.removecomponent,'Enable','on','Visible','on')

name = {get(data.dynamics.temp.addcontroller.Name,'String')};
allNames = get(data.handles.controllerindex,'String');
for i = 1:1:length(allNames)
    if(strcmp(name,allNames{i}))
        name = strcat(data.designcontrollers,'_',num2str(data.DesignNaming));
        data.DesignNaming = data.DesignNaming + 1;
    end
end
set(data.handles.controllerindex,'String',[get(data.handles.controllerindex,'String'); name])

set(data.handles.controllerindex,'Value',length(get(data.handles.controllerindex,'String')))

set(data.handles.plotcontroller,'Value',1);

num = data.dynamics.temp.addcontroller.Numerator;
den = data.dynamics.temp.addcontroller.Denominator;

set(data.handles.numeratorcontroller,'String',num)
set(data.handles.denominatorcontroller,'String',den)
set(data.handles.showcomponents,'String','Gain');
data.CurController = length(data.symController) + 1;

data.Design{data.CurController}.num = num;
data.Design{data.CurController}.den = den;

data.Design{data.CurController}.Components = {[1],'1'};

data.Design{data.CurController}.ComponentsNum = {'1'};
data.Design{data.CurController}.ComponentsDen = {'1'};

data.plotController(data.CurController) = 1;
set(data.handles.gainedit,'Visible','on','Enable','on','String','1')
set(data.handles.gaintext,'Visible','on','Enable','on')
set(data.handles.unitgain,'Visible','on','Enable','on')
data.Design{data.CurController}.vecElements{1} = [0 0 0 1 1 1 1 1 1 1 1 1 1 1 1];
data.Design{data.CurController}.Name = name;

data.Design{data.CurController}.ControllerStable = data.Design{1}.ControllerStable;

setEdit(1);
data.Design{data.CurController}.gainValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.gainedit,'String');
data.Design{data.CurController}.zeroValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.zeroedit,'String');
data.Design{data.CurController}.poleValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.poleedit,'String');
data.Design{data.CurController}.dampzeroValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.dampzeroedit,'String');
data.Design{data.CurController}.damppoleValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.damppoleedit,'String');

closePopUp(data.dynamics.temp.h);

data.Controller{data.CurController} = data.Controller{1};
data.symController{data.CurController} = data.symController{1};

plotController();

set(data.handles.savecontroller,'Enable','on');

return

%%
%       function createOwnController
%       
%       This function allows the user to create its own controller via a
%       transfer function or load it via the workspace. 
%
%%
function createOwnController(varargin)
global data

closePopUp(data.dynamics.temp.h);

Title = data.ListControllers{1};
if(length(data.ListControllers)>1)
    for i = 2:1:length(data.ListControllers)
        Title = [Title,' = ',data.ListControllers{i}];
    end
end

h = figure;
data.dynamics.temp.h = h;
hDesiredSize = [520, 160];
set(h,'Resize','off','MenuBar','none','ToolBar','none','NumberTitle','off','Name',...
    ['Design Controller ',Title],'Position',centerPopUp(hDesiredSize),...
    'Units','normalized','Color',[.9 .9 .9],'WindowStyle','modal');

% Declaration Cancel button
hcancel = uicontrol(h,'Style','pushbutton','String','Cancel','Units','normalized',...
    'Position',[.25 .06 .2 .2],'Backgroundcolor',data.colourbutton,'FontSize',[11],...
    'Callback','global data; closePopUp(data.dynamics.temp.h); clear data;');

% Declaration Define button
hdefine = uicontrol(h,'Style','pushbutton','String','Define','Units','normalized',...
    'Position',[.55 .06 .2 .2],'Backgroundcolor',data.colourbutton,'FontSize',[11],...
    'Callback',@definecontroller);

uicontrol(h,'Style','text','String','Numerator',...
    'Units','normalized','Position',[.35 .65 .3 .3],'Backgroundcolor',...
    [.9 .9 .9]);
uicontrol(h,'Style','text','String','Denominator',...
    'Units','normalized','Position',[.35 .35 .3 .3],'Backgroundcolor',...
    [.9 .9 .9]);

uicontrol(h,'Style','text','String','Name','Units','normalized',...
    'Position',[.1 .70 .1 .1],'Backgroundcolor',[.9 .9 .9]);
data.dynamics.temp.addcontroller.Name = uicontrol(h,'Style','edit',...
    'String',[data.ListControllers{1},'_',num2str(data.DesignNaming)],...
    'Units','normalized','Backgroundcolor',[1 1 1],'Position',[.05 .55 .2 .15]);

data.dynamics.temp.addcontroller.Numerator = uicontrol(h,'Style','edit','String',num2str(1),...
    'Units','normalized','UserData',num2str(1),...
    'Backgroundcolor',[1 1 1],'Position',[.3 .7 .4 .15]);

data.dynamics.temp.addcontroller.Denominator = uicontrol(h,'Style','edit','String',num2str(1),'UserData',num2str(1),...
    'Units','normalized','Backgroundcolor',[1 1 1],'Position',[.3 .4 .4 .15]);

uicontrol(h,'Style','pushbutton','String',...
    'From Workspace','units','normalized','Position',[.72 .7 .25 .15],...
    'Callback',@loadfromworkspace);
uicontrol(h,'Style','pushbutton','String',...
    'From File','units','normalized','Position',[.72 .55 .25 .15],...
    'Callback',@loadfromfile);

data.dynamics.temp.Name = 'addcontroller';
data.dynamics.temp.Abbr = 'addcontroller';

return

%%
%       function definecontroller
%       
%       This function saves the controller that is created by the user
%       itself via transfer function
%
%%
function definecontroller(varargin)
global data

if(isstr(data.dynamics.temp.addcontroller.Numerator))
    num = data.dynamics.temp.addcontroller.Numerator;
    den = data.dynamics.temp.addcontroller.Denominator;
else
    num = get(data.dynamics.temp.addcontroller.Numerator,'String');
    den = get(data.dynamics.temp.addcontroller.Denominator,'String');
end
name = {get(data.dynamics.temp.addcontroller.Name,'String')};

if(~isempty(strfind(name,data.designcontrollers)))
    name = strcat(data.designcontrollers,'_',num2str(data.DesignNaming));
    data.DesignNaming = data.DesignNaming + 1;
end
set(data.handles.controllerindex,'String',[get(data.handles.controllerindex,'String'); name])
set(data.handles.numeratorcontroller,'String',num)
set(data.handles.denominatorcontroller,'String',den)
set(data.handles.removecomponent,'Visible','on','Enable','on');

data.CurController = length(data.symController) + 1;
data.Design{data.CurController}.num = num;
data.Design{data.CurController}.den = den;
data.Design{data.CurController}.Components = {};
data.Design{data.CurController}.Name = name;
data.Design{data.CurController}.ControllerStable = {};

evaluateDefinedController('on')

set(data.handles.savecontroller,'Enable','on');

closePopUp(data.dynamics.temp.h);

return

%%
%       function evaluateDefinedController
%       
%       This function evaluates the stability properties of this
%       controller and gives the frequency range for which it is
%       stable/unstable.
%
%%
function evaluateDefinedController(action)
global data;

n = data.CurController;

num = data.Design{n}.num;
den = data.Design{n}.den;

if(~isempty(data.Design{n}.Components))
    for i = 1:1:length(data.Design{n}.Components{1})
        num = strcat('(',num,')*(',data.Design{n}.Components{i+1},')');
        [f1 f2 b1 b2 P D] = getEdit(i);
        num = subsComp(num, f1,f2, b1, b2, P ,D);
        den = subsComp(den, f1,f2, b1, b2, P ,D);
    end
end

num = strrep(num,'*','.*');
den = strrep(den, '*', '.*');
num = strrep(num,'/','./');
den = strrep(den, '/', './');
num = strrep(num,'^','.^');
den = strrep(den, '^', '.^');

syms s
data.symController{n} = (eval(num))./(eval(den));

if(data.plot.unitsFreq.Value == 1)
    s = 1j.*2.*pi.*data.parameters.Freq;
else
    s = 1j.*data.parameters.Freq;
end
C = eval(num)./ eval(den);
if(length(C) == length(data.parameters.Freq))
    data.Controller{n} = C;
elseif(length(C) == 1)
    data.Controller{n} = ones(size(data.parameters.Freq))*C;
else
    errorHandling('Something goes wrong here');
end

data.plotController(data.CurController) = 1;

set(data.handles.controllerindex,'Value',length(get(data.handles.controllerindex,'String')))

set(data.handles.numeratorcontroller,'Enable',action)
set(data.handles.denominatorcontroller,'Enable',action)

set(data.handles.plotcontroller,'Value',1);

evaluateStability('final');

plotController();

return

%%
%       function addcomponent
%       
%       This function allows the user to add components
%
%%
function addcomponent(action)
global data

set(data.handles.showcomponents,'String',...
    [get(data.handles.showcomponents,'String'); {action}])

strElements = {'gainedit','gaintext','unitgain','zerotext','zeroedit','unitzero',...
    'poletext','poleedit','unitpole','dampzerotext','dampzeroedit','unitdampzero',...
    'damppoletext','damppoleedit','unitdamppole'};

Components = data.Design{data.CurController}.Components;


switch action
    case 'Gain'
        % on
        set(data.handles.gainedit,'Visible','on','Enable','on','String','1')
        set(data.handles.gaintext,'Visible','on','Enable','on')
        set(data.handles.unitgain,'Visible','on','Enable','on')
        % off
        vecElements = [0 0 0 1 1 1 1 1 1 1 1 1 1 1 1];
        
        index = 1;
        num = '1';
        den = '1';
        C = '1';
        
    case 'Integrator'
        set(data.handles.zerotext,'Enable','on','Visible','on','String','zero')
        set(data.handles.zeroedit,'Enable','on','Visible','on','String','1')
        set(data.handles.unitzero,'Enable','on','Visible','on','String',data.plot.unitsFreq.String)
        vecElements = [1 1 1 0 0 0 1 1 1 1 1 1 1 1 1];
        
        num = 's+f1';
        den = 's';
        
        index = 2;
        C = '(s+f1)/s';
        
    case 'Lead/lag'
        set(data.handles.poletext,'Enable','on','String','pole')
        set(data.handles.poleedit,'Enable','on','String','10')
        set(data.handles.unitpole,'Enable','on','String',data.plot.unitsFreq.String)
        
        set(data.handles.zerotext,'Enable','on','String','zero')
        set(data.handles.zeroedit,'Enable','on','String','1')
        set(data.handles.unitzero,'Enable','on','String',data.plot.unitsFreq.String)
        
        vecElements = [ 1 1 1 0 0 0 0 0 0 1 1 1 1 1 1];
        num = 'f2*(s+f1)';
        den = 'f1*(s+f2)';
        
        index = 3;
        C = 'f2*(s+f1)/(f1*(s+f2))';
        
    case 'Low pass 1st order'
        set(data.handles.poletext,'Enable','on','String','pole')
        set(data.handles.poleedit,'Enable','on','String','10')
        set(data.handles.unitpole,'Enable','on','String',data.plot.unitsFreq.String)
        
        vecElements = [ 1 1 1 1 1 1 0 0 0 1 1 1 1 1 1];
        
        num = 'f2';
        den = 's+f2';
        
        index = 4;
        C = 'f2/(s+f2)';
        
    case 'Low pass 2nd order'
        set(data.handles.poletext,'Enable','on','String','poles')
        set(data.handles.poleedit,'Enable','on','String','10')
        set(data.handles.unitpole,'Enable','on','String',data.plot.unitsFreq.String)
        
        set(data.handles.damppoletext,'Enable','on')
        set(data.handles.damppoleedit,'Enable','on','String','.7')
        set(data.handles.unitdamppole,'Enable','on')
        
        vecElements = [ 1 1 1 1 1 1 0 0 0 1 1 1 0 0 0];
        
        num = '(f2)^2';
        den = 's^2+b2*f2+(f2)^2';
        index = 5;
        C = '(f2)^2/(s^2+b2*f2+(f2)^2)';
        
    case 'Notch'
        set(data.handles.zerotext,'Enable','on','String','zero')
        set(data.handles.zeroedit,'Enable','on','String','10')
        set(data.handles.unitzero,'Enable','on','String',data.plot.unitsFreq.String)
        
        set(data.handles.dampzerotext,'Enable','on')
        set(data.handles.dampzeroedit,'Enable','on','String','.1')
        set(data.handles.unitdampzero,'Enable','on')
        
        set(data.handles.poletext,'Enable','on','String','poles')
        set(data.handles.poleedit,'Enable','on','String','10')
        set(data.handles.unitpole,'Enable','on','String',data.plot.unitsFreq.String)
        
        set(data.handles.damppoletext,'Enable','on')
        set(data.handles.damppoleedit,'Enable','on','String','.5')
        set(data.handles.unitdamppole,'Enable','on')
        
        vecElements = [ 1 1 1 0 0 0 0 0 0 0 0 0 0 0 0];
        
        num = '(f2)^2*(s^2+b1*f1+(f1)^2)';
        den = '(f1)^2*(s^2+b2*f2+(f2)^2)';
        index = 6;
        C = '((f2)^2*(s^2+b1*f1+(f1)^2))/((f1)^2*(s^2+b2*f2+(f2)^2))';
        
    case 'PD'
        set(data.handles.poletext,'Enable','on','String','D')
        set(data.handles.poleedit,'Enable','on','String','1')
        set(data.handles.unitpole,'Enable','on','String','-')
        
        set(data.handles.zerotext,'Enable','on','String','P')
        set(data.handles.zeroedit,'Enable','on','String','1')
        set(data.handles.unitzero,'Enable','on','String','-')
        
        vecElements = [ 1 1 1 0 0 0 0 0 0 1 1 1 1 1 1];
        
        num = 'f1+f2*s';
        den = '1';
        index = 7;
        C = 'f1+f2*s';
        
    case 'add'
        errorHandling('Cannot Be!');
        return
end

for i = 1:1:length(strElements)
    if(vecElements(i) == 1)
        set(data.handles.(strElements{i}),'Enable','off','Visible','off');
    else
        set(data.handles.(strElements{i}),'Visible','on');
    end
end

data.Design{data.CurController}.Components{1} = [Components{1},index];
data.Design{data.CurController}.Components{length(data.Design{data.CurController}.Components)+1} = C;
data.Design{data.CurController}.vecElements{length(data.Design{data.CurController}.Components{1})} = vecElements;

data.Design{data.CurController}.ComponentsNum{length(data.Design{data.CurController}.Components{1})} = num;
data.Design{data.CurController}.ComponentsDen{length(data.Design{data.CurController}.Components{1})} = den;

data.Design{data.CurController}.gainValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.gainedit,'String');
data.Design{data.CurController}.zeroValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.zeroedit,'String');
data.Design{data.CurController}.poleValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.poleedit,'String');
data.Design{data.CurController}.dampzeroValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.dampzeroedit,'String');
data.Design{data.CurController}.damppoleValue{length(data.Design{data.CurController}.Components)-1} =...
    get(data.handles.damppoleedit,'String');

[f1 f2 b1 b2 P D] = getEdit(length(data.Design{data.CurController}.Components)-1);

repNum = subsComp(num, f1,f2, b1, b2, P ,D);

set(data.handles.numeratorcontroller,'String',repNum);

repDen = subsComp(den, f1,f2, b1, b2, P ,D);

set(data.handles.denominatorcontroller,'String',repDen);

set(data.handles.showcomponents,'Value',length(get(data.handles.showcomponents,'String')))
set(data.handles.addcomponent,'Value',1,'Enable','on','Visible','on')

if(index == 1)
    designController('gain');
else
    designController('zero');
end

return

%%
%       function removeComponent
%       
%       This function allows the user to remove components
%
%%
function removeComponent(remove)
global data

switch remove
    case 'Design'
        if(data.CurController ~= 1)
            set(data.plot.Magcontroller{data.CurController},'Visible','off')
            set(data.plot.Phasecontroller{data.CurController},'Visible','off')
            set(data.plot.Pointcontroller{data.CurController},'Visible','off')
            
            data.plot.Magcontroller{data.CurController} = {};
            data.plot.Phasecontroller{data.CurController} = {};
            data.plot.Pointcontroller{data.CurController} = {};
            Design = data.Design;
            Controller = data.Controller;
            symController = data.symController;
            data.Design = {};
            data.Controller = {};
            data.symController = {};
            j = 1;
            for i = 1:1:length(Design)
                if(i ~= data.CurController)
                    data.Design{j} = Design{i};
                    data.symController{j} = symController{i};
                    data.Controller{j} = Controller{i};
                    j = j + 1;
                end
            end
            ControllerList = get(data.handles.controllerindex,'String');
            j = 1;
            for i = 1:1:length(ControllerList)
                if(~strcmp(ControllerList{i},ControllerList{get(data.handles.controllerindex,'Value')}))
                    newControllerList{j} = ControllerList{i};
                    j = j + 1;
                end
            end
            set(data.handles.controllerindex,'Value',1,'String',newControllerList);
            
            j = 1;
            plotController = data.plotController;
            data.plotController = zeros(1,length(plotController)-1);
            
            for i = 1:1:length(plotController)
                if(i ~= data.CurController)
                    data.plotController(j) = plotController(i);
                    j = j + 1;
                end
            end
            
            set(data.handles.showcomponents,'String','Gain','Value',1)
            
            designController('choosecontroller');
        end
    case 'Component'
        if(~isempty(data.Design{data.CurController}.Components))
            ComponentList = get(data.handles.showcomponents,'String');
            toRemoveComp = get(data.handles.showcomponents,'Value');
            
            j = 1;
            Components = data.Design{data.CurController}.Components;
            numComp = data.Design{data.CurController}.ComponentsNum;
            denComp = data.Design{data.CurController}.ComponentsDen;
            vecElements = data.Design{data.CurController}.vecElements;
            gainValue = data.Design{data.CurController}.gainValue;
            zeroValue = data.Design{data.CurController}.zeroValue;
            poleValue = data.Design{data.CurController}.poleValue;
            dampzeroValue = data.Design{data.CurController}.dampzeroValue;
            damppoleValue = data.Design{data.CurController}.damppoleValue;
            noComp = length(Components{1});
            
            if(noComp > 1)
                data.Design{data.CurController}.Components = {};
                data.Design{data.CurController}.ComponentsNum = {};
                data.Design{data.CurController}.ComponentsDen = {};
                data.Design{data.CurController}.vecElements = {};
                data.Design{data.CurController}.gainValue = {};
                data.Design{data.CurController}.zeroValue = {};
                data.Design{data.CurController}.poleValue = {};
                data.Design{data.CurController}.dampzeroValue = {};
                data.Design{data.CurController}.damppoleValue = {};
                for i = 1:1:noComp
                    if(i ~= toRemoveComp)
                        newComponentList{j} = ComponentList{i};
                        data.Design{data.CurController}.Components{1}(j) =...
                            Components{1}(i);
                        data.Design{data.CurController}.Components{j+1} =...
                            Components{i+1};
                        data.Design{data.CurController}.ComponentsNum{j} =...
                            numComp{i};
                        data.Design{data.CurController}.ComponentsDen{j} =...
                            denComp{i};
                        data.Design{data.CurController}.vecElements{j} =...
                            vecElements{i};
                        data.Design{data.CurController}.gainValue{j} =...
                            gainValue{i};
                        data.Design{data.CurController}.zeroValue{j} =...
                            zeroValue{i};
                        data.Design{data.CurController}.poleValue{j} =...
                            poleValue{i};
                        data.Design{data.CurController}.dampzeroValue{j} =...
                            dampzeroValue{i};
                        data.Design{data.CurController}.damppoleValue{j} =...
                            damppoleValue{i};
                        j = j + 1;
                    end
                end
                set(data.handles.showcomponents,'String',newComponentList,'Value',1);
                
                set(data.plot.Magcontroller{data.CurController},'Visible','off')
                set(data.plot.Phasecontroller{data.CurController},'Visible','off')
                set(data.plot.Pointcontroller{data.CurController},'Visible','off')
                
                data.plot.Magcontroller{data.CurController} = {};
                data.plot.Phasecontroller{data.CurController} = {};
                data.plot.Pointcontroller{data.CurController} = {};
               
                designController('showcomponents');
                evaluateDefinedController('off');
%              
            end
        end
        
end

if(size(get(data.handles.controllerindex,'String')) <= 1)
    set(data.handles.savecontroller,'Enable','off');
else
    set(data.handles.savecontroller,'Enable','on');
end

set(data.handles.removecomponent,'Value',1);
set(data.handles.numeratorcontroller,'Enable','off');
set(data.handles.denominatorcontroller,'Enable','off');


return

%%
%       function setEdit
%       
%       This function sets all the editboxes (gain,zero,pole,damp) to
%       desired format
%
%%
function setEdit(index)
global data

% Some editors should be visible and other should not.
for i = 1:1:length(data.strElements)
    if( data.Design{data.CurController}.vecElements{index}(i) == 1)
        set(data.handles.(data.strElements{i}),'Enable','off','Visible','off');
    else
        set(data.handles.(data.strElements{i}),'Visible','on','Enable','on');
    end
end
set(data.handles.unitpole,'String',data.plot.unitsFreq.String{1});
set(data.handles.unitzero,'String',data.plot.unitsFreq.String{1});

return


%%
%       function loadExistingController
%       
%       This function allows the loading of a controller that has already
%       been defined using the FMBS tool
%
%%
function loadExistingController(varargin)
global data

[filename, pathname] = uigetfile('*.mat', 'Pick a .MAT file');

if(filename == 0)
    otherButtons('enable');
    set(data.handles.defineElement,'string',''); drawnow;
    
    return
end

load(fullfile(pathname,filename));

try
    data.CurController = length(data.Design) + 1;
    data.Design{data.CurController} = controller;
    
    set(data.handles.controllerindex,'String',[get(data.handles.controllerindex,'String');...
        data.Design{data.CurController}.Name],'Value',data.CurController)
    set(data.handles.numeratorcontroller,'String',data.Design{data.CurController}.num);
    set(data.handles.denominatorcontroller,'String',data.Design{data.CurController}.den);
    set(data.handles.removecomponent,'Visible','on','Enable','on');
    if(isempty(data.Design{data.CurController}.Components))
        set(data.handles.numeratorcontroller,'Enable','on')
        set(data.handles.denominatorcontroller,'Enable','on')
    else
        set(data.handles.numeratorcontroller,'Enable','off')
        set(data.handles.denominatorcontroller,'Enable','off')
        Components = get(data.handles.addcomponent,'String');
        String = {Components{data.Design{data.CurController}.Components{1}(1)+1}};
        if(length(data.Design{data.CurController}.Components{1}) > 1)
            for i = 2:1:length(data.Design{data.CurController}.Components{1})
                String = [String; Components{data.Design{data.CurController}.Components{1}(i)+1}];
            end
        end
        
        set(data.handles.showcomponents,'Visible','on','Enable','on','Value',1,...
            'String',String);
        setEdit(1)
        set(data.handles.gainedit,'String',data.Design{data.CurController}.gainValue{1});
        set(data.handles.zeroedit,'String',data.Design{data.CurController}.zeroValue{1});
        set(data.handles.poleedit,'String',data.Design{data.CurController}.poleValue{1});
        set(data.handles.dampzeroedit,'String',data.Design{data.CurController}.dampzeroValue{1});
        set(data.handles.damppoleedit,'String',data.Design{data.CurController}.damppoleValue{1});
        set(data.handles.unitzero,'String',data.plot.unitsFreq.String{1});
        set(data.handles.unitpole,'String',data.plot.unitsFreq.String{1});
        
        set(data.handles.removecomponent,'Visible','on','Enable','on');
        
        if(data.Design{data.CurController}.Components{1}(1) == 1)
            set(data.handles.numeratorcontroller,'String', strcat('(',...
                data.Design{data.CurController}.gainValue{1},')*(',...
                data.Design{data.CurController}.num,')'));
            set(data.handles.denominatorcontroller,'String',data.Design{data.CurController}.den);
        else
            [f1 f2 b1 b2 P D] = getEdit(1);
            num = data.Design{data.CurController}.ComponentsNum{1};
            den = data.Design{data.CurController}.ComponentsDen{1};
            repNum = subsComp(num,f1, f2, b1, b2, P, D);
            repDen = subsComp(den,f1, f2, b1, b2, P, D);
            set(data.handles.numeratorcontroller,'String',repNum);
            set(data.handles.denominatorcontroller,'String',repDen);
        end
    end
    
    evaluateDefinedController('off')
    
    set(data.handles.savecontroller,'Enable','on');
    
catch
    errorHandling('Only include previously saved controllers')
    return
end

closePopUp(data.dynamics.temp.h);

return












