%%
%   function startMapping computes matrices with the evaluated stability
%   criterion and frequency, magnitude and phase combinations.
%
%%
function startMapping(action,hObject,eventdata,handles)
global data

data.mapped = 1;

set(data.handles.plot3ddiagram,'Enable','on');
set(data.handles.savemapping,'Enable','on');
set(data.handles.designcontroller,'Enable','on');
set(data.handles.plotcontroller,'Enable','on','Value',1);
data.CurController = 1;
data.plotController(data.CurController) = 1;
data.Design{data.CurController}.num = data.dynamics.(data.designcontrollers).Numerator;
data.Design{data.CurController}.den = data.dynamics.(data.designcontrollers).Denominator;
data.Design{data.CurController}.ControllerStable = {};

set(data.handles.plotfrequencyin,'Enable','on');
set(data.handles.plotmagnitudein,'Enable','on');
set(data.handles.plotphasein,'Enable','on');
set(data.handles.MagPhaseperSlider,'Enable','on');
set(data.handles.controllerindex,'Enable','on');
set(data.handles.rectangles,'Enable','on','Value',0);
set(data.handles.showinbode,'Enable','on','Value',0);
set(data.handles.removecomponent,'Enable','on','Visible','on');
data.plot.rectanglesFlag = 0;
data.plot.showinbodeFlag = 0;
data.plot.Magcontroller = {};
data.plot.Phasecontroller = {};
data.plot.Pointcontroller = {};
data.plot.Data.MagPerFreq2D = [];
data.plot.Data.PhasePerFreq2D = [];
data.plot.Data.FreqMag2D = {};
data.plot.Data.FreqPhase2D = {};

otherButtons('disable');
data.stepOfAction(1,2) = 8;
data.stepOfAction(1,1) = 1;

set(data.handles.defineElement,'string',...
    ['Stability Mapping: Extract Dynamics (Step ',num2str(data.stepOfAction(1,1)),' of ',...
    num2str(data.stepOfAction(1,2)),').']); drawnow;
data.stepOfAction(1,1) = data.stepOfAction(1,1) + 1;

% definedynamics('definitionCheck',hObject,eventdata,handles);

% Various checks whether it is possible:
switch data.stabilitycriterion
    case 'Default'
        errorHandling('Stability Criterion is not yet defined');
        otherButtons('enable');
        return
end

try
    mappingComputation('normal',action,hObject,eventdata,handles);

catch ME
    idSegLast = regexp(ME.identifier, '(?<=:)\w+$', 'match');
    %clean up for possible error conditions here. Rethrow if unknown error.
    switch idSegLast{1}
        case 'nomem'
        %out of memory
        errorHandling('Out of memory, so choose less points');
        case 'couldNotReadFile'
        %file not there
        errorHandling('Some Error Occurs, you use too many points or you found a bug!')
        otherwise
        %An unexpected error happened
        errorHandling('Some Error Occurs, you use too many points or you found a bug!')
    end
end

return

function mappingComputation(computationSetting,action,hObject,eventdata,handles)
global data

switch computationSetting
    case 'normal'
        % stability criterion, dynamics, controllers
        extractDynamics();        
                
        %% Evaluate Stability Controller
        set(data.handles.defineElement,'string',['Evaluate Stability (Step ',...
        num2str(data.stepOfAction(1,1)),' of ',num2str(data.stepOfAction(1,2)),').']); drawnow;
        data.stepOfAction(1,1) = data.stepOfAction(1,1) + 1;
                
        data.plot.unitsFreq = data.discretisation.unitsFreq;
        data.plot.unitsMag = data.discretisation.unitsMag;
        data.plot.unitsPhase = data.discretisation.unitsPhase;
        
        evaluateStability('final');
        
        set(data.handles.defineElement,'string',...
            ['Stability Mapping: Evaluate Stability (Step ',num2str(data.stepOfAction(1,1)),' of ',...
            num2str(data.stepOfAction(1,2)),').']); drawnow;
        data.stepOfAction(1,1) = data.stepOfAction(1,1) + 1;
        evaluateStability('matrices');       
               
        if(data.discretisation.discretisationmethod == 1)
            set(data.handles.defineElement,'string',...
                ['Advanced Mapping: (Step ',num2str(data.stepOfAction(1,1)),' of ',...
                num2str(data.stepOfAction(1,2)),').']); drawnow;
            data.stepOfAction(1,1) = data.stepOfAction(1,1) + 1;
            
            mapping('advanced');
        else
            set(data.handles.defineElement,'string',...
                ['Stability Mapping: Creating 2D Bode Diagram (Step ',num2str(data.stepOfAction(1,1)),' of ',...
                num2str(data.stepOfAction(1,2)),').']); drawnow;
            data.stepOfAction(1,1) = data.stepOfAction(1,1) + 1;
            
            mapping('2D')
        end

        %% pLOT
        plotDiagram(action,hObject,eventdata,handles);
        
        set(data.handles.defineElement,'string',''); drawnow;
        
        otherButtons('enable');
        
        if(data.stepOfAction(1,1) == data.stepOfAction(1,2)+1)
            data.stepOfAction(1,1) = 1;
        else
            errorHandling('Stepcount is wrong!')
            data.stepOfAction(1,1) = 1;
        end
        
    case 'memoryError'
        otherButtons('enable');
        data.stepOfAction(1,1) = 1;       
        
end

return


%%
%   function extractDynamics
%
%%

function extractDynamics()
global data;

dyn  = data.dynamics;

%% Extract w, M and P vectors from discretisation settings
createVectors(1);

Freq = data.parameters.Freq;
Mag = data.parameters.Mag;
Phase = data.parameters.Phase;

nFreq = length(Freq);
nMag = length(Mag);
nPhase = length(Phase);

%% Create Impedance-matrices Z (1x4) and Controllers C (1x8)
nZ = 4;
nC = 8;
Z = cell(1,nZ);
C = cell(1,nC);

for i = 1:1:nZ
    Z{i} = zeros(nFreq,nMag,nPhase);
end

Cvar{1} = 1;

switch data.designcontrollers
    case {'C1','C2','C3','C4','C5','C6'}
        Cvar{1} = str2num(data.designcontrollers(2));
    case 'Cs'
        Cvar{1} = 7;
    case 'Cm'
        Cvar{1} = 8;
end
j = 2;

for i = 1:1:nC
    C{i} = zeros(nFreq,nMag,nPhase);
    if(strfind(dyn.(dyn.elements{i+4}).otherControllerString,data.designcontrollers) ~= 0)
        if(dyn.(dyn.elements{i+4}).otherControllerIndex == 1)
            % Controller is function of other controller
            Cvar{j} = i;
            j = j + 1;
        end
    end
end

data.ListControllers = {};
for i = 1:1:j-1
    data.ListControllers{i} = dyn.elements{Cvar{i}+4};
end

n = length(dyn.elements);

%% Fill in matrices Z,C

data.dynamics.Cvar = Cvar;
dyn.Cvar = Cvar;
[numZ,denZ,numC,denC] = fixDynamics(1);

for iW = 1:1:nFreq
    w = Freq(iW);
    s = 1j*w;
    for iC = 1:1:n
        if( iC <= nZ )
            %Element is impedance
            Z{iC}(iW,:,:) = (eval(numZ{iC}))/(eval(denZ{iC}));
        else
            %Element is controller
            
            variable = (Mag'*cos(Phase) + 1j*Mag'*sin(Phase));
            C{iC-4}(iW,:,:) = (eval(numC{iC-nZ}))/(eval(denC{iC-nZ}));
            
        end
    end
    
end
S = repmat(1j*Freq',[1,nMag,nPhase]);

dyn.C = C;
dyn.Z = Z;
dyn.S = S;

data.dynamics = dyn;

createVectors(0)

return

function createVectors(flag)
global data;

disc = data.discretisation;

%% Extract w, M and P vectors from discretisation settings
nFreq  = disc.nFreq;
nMag   = disc.nMag;
nPhase = disc.nPhase;

if(flag)
    % Create vectors.
    % (Freq: logarithmic vector in rad/s)
    switch disc.unitsFreq.Value
        case 1        %HZ
            minF = log10(disc.minFreq*2*pi);
            maxF = log10(disc.maxFreq*2*pi);
        case 2        %rad/s
            minF = log10(disc.minFreq);
            maxF = log10(disc.maxFreq);
    end
    Freq  = logspace(minF,maxF,nFreq);
    
    % (Mag: dB)
    Mag   = 10.^(linspace(disc.minMag,disc.maxMag,nMag)./20);
    
    % (Phase: rad)
    switch disc.unitsPhase.Value
        case 1        % degree
            minP = disc.minPhase/360*2*pi;
            maxP = disc.maxPhase/360*2*pi;
        case 2        % rad
            minP = disc.minPhase;
            maxP = disc.maxPhase;
    end
    Phase = linspace(minP,maxP,nPhase);
    
    data.parameters.tempUnitFreq = 'rad/s';
    data.parameters.tempUnitMag = 'dB';
    data.parameters.tempUnitPhase = 'rad';
else
    % Reset vectors.
    % (Freq: logarithmic vector is in rad/s)
    
    Freq = data.parameters.Freq;
    Mag = data.parameters.Mag;
    Phase = data.parameters.Phase;
    
    if(disc.unitsFreq.Value == 1)%Hz
        Freq = Freq./(2.*pi);
    end
    
    % (Mag: dB)
    Mag   = db(Mag);
    
    % (Phase: rad/s
    if (disc.unitsPhase.Value == 1)% degree
        Phase = Phase.*360./(2.*pi);
    end
    
    data.parameters.tempUnitFreq = disc.unitsFreq.String;
    data.parameters.tempUnitMag = disc.unitsMag.String;
    data.parameters.tempUnitPhase = disc.unitsPhase.String;
end

data.parameters.Freq = Freq;
data.parameters.Mag = Mag;
data.parameters.Phase = Phase;

return

